@@ -11,7 +11,7 @@ trap "cleanup" EXIT
 
 COMPLETION_ROOT_REL="contrib/completions"
 COMPLETION_ROOT="${OS_ROOT}/${COMPLETION_ROOT_REL}"
-TMP_COMPLETION_ROOT_REL="_output/verify-generated-completions/"
+TMP_COMPLETION_ROOT_REL="_output/verify-generated-completions"
 TMP_COMPLETION_ROOT="${OS_ROOT}/${TMP_COMPLETION_ROOT_REL}"
 
 ${OS_ROOT}/hack/update-generated-completions.sh ${TMP_COMPLETION_ROOT_REL}
